/**************/
/* checker.c  */
/*****************************************************************************
    MODULE:  checker.c
    PURPOSE: Aliohjelmakirjasto moniajon toteuttamiseksi.
             (Tm on viel pahasti prototyyppiasteella!)
    AUTHOR:  Vesa Lappalainen 10.9.1992 ja tulostus 30.10.1992
             siirretty omaksi tiedostoksi 3.11.1992

    USAGE:   Kirjaston kyttmiseksi tarvitaan projektiin tiedostot:
                 ALI\checker.c
                 ALI\tulostus.c      - jos halutaan tulostaa!
             ja #includeen:
                 chekcer.h
                 tulostus.h      - jos halutaan tulostaa!

             Jollei tulostusta haluta, pit mritell vakio NOT_PRINTER

    Aliohjelmakirjaston kyttmiseksi pit ohjelmoijan tehd ohjelmaansa
    seuraavat muutokset:

      1) listtv piirron pitkiin silmukoihin kutsu
      ----------------------------------------------
             if ( CheckMessage() ) lopetetaan_piirto...;

      2) listtv ikkunafunktioon ennen muiden viestien ksittely koodi:
      ---------------------------------------------------------------------
             CHECK_CHECKER_WIN(hWnd,message,wParam,lParam);
         tai dialogin ikkunafunktioon vastaavasti:
             CHECK_CHECKER_DIALOG(hWnd,message,wParam,lParam);
         HUOM!  Nm ovat makroja, jotka saattavat poistua ikkunafunktiosta
                heti samasta paikasta.    

      3) lopettettava piirtminen EndPaint viestin sijasta kutsulla
      -------------------------------------------------------------
             CheckEndPaint(hWnd,&ps);

      4) listtv viestisilmukan loppuun
      -----------------------------------
             while (GetMessage(&msg,NULL,NULL,NULL)) {
                 TranslateMessage(&msg);
                 DispatchMessage(&msg);
                 CheckDelayMessages(hWnd);   /* Moniajo tarvitsee!
                 -------------------------
             }

         (CheckDelayMessages(hWnd) sijaan voidaan kutsua mys
          CheckEndPaint(hWnd,NULL), joka tekee saman asian.  Tarvitaan
          jos on tiedosssa vain CheckEndPaintin osoite, esim tabhand.c)

    Tulostuksen takia moniajon tarkistuksessa pit valvoa mys
    GlobalPrinter.Printing -lippua, jottei rekursiivisesti kutsuta
    piirto-ohjelmaa!

    Tst seuraa se, ett mys WM_PAINT -viesti pit muuttaa viivstytesti
    suoritettavaksi.

    Jos ohjelma yritetn sulkea kesken toiminnon, j sulkeminen odottamaan
    kunnes toiminto on valmis TAI saatu keskeytetyksi!
    Viivstytetyst sulkemisesta tulee viesti kyttjlle ikkunan
    otsikkoon.

============
Aliohjelmat: (checker.c, checker.h)
============
  CheckMessage()                                - halutaanko keskeytt
  IsCheckingMessage(hWnd,message,wParam,lParam) - onko keskeyttjn viesti
  CheckEndPaint(hWnd,&ps)                       - lopetetaan piirto
  CheckDelayMessages(hWnd)                      - onko viivstettyj viestej

=============================
Aliohjelmien tarkempi kuvaus:
=============================
  ----------------------
  int CheckMessage(void) - halutaanko keskeytt
  ----------------------
    Tm funktio antaa aikaa muille ohjelmille suoritusta varten.  Samalla
    se tutkii pitk piirtminen lopettaa vai ei.

    Palauttaa:
       0 = ei haluta keskeytt
       1 = halutaan keskeytt

  -----------------------------------------------------------------------------
  int IsCheckingMessage(HWND hWnd,unsigned message,WPARAM wParam,LPARAM lParam)
  -----------------------------------------------------------------------------
    Funktiolla tutkitaan kuuluuko viesti keskeyttjlle ja jos kuuluu, niin
    miten viesti on ksiteltv jatkossa.

    Palauttaa:
       0 = viesti ei kuulu keskeyttjlle ja ksitelln normaalisti
           omassa ikkunafunktiossa
       1 = viesti kuului keskeyttjlle ja se on ksitelty tysin
           ikkunafunktion kuuluu palauttaa ett viesti on ksitelty
           (NULL ikkunafunktiossa ja TRUE dialogin ikkunafunktiossa)
      -1 = viesti kuului keskeyttjlle ja se pit antaa oletusksittelijlle
           jatkossa
           (DefWindowProc ikkunafunktiossa ja FALSE dialogissa)

  ----------------------------
  void CheckEndPaint(hWnd,&ps) - lopetetaan piirto
  ----------------------------
    Tt funktiota tytyy kutsua EndPaint -funktion tilalla, jotta keskeyttj
    tiet onko piirtminen saatu rauhassa tehd loppuun vaiko ei.  Tmn
    perusteella mrtn osaltaa seuraavien piirtojen piirtoaluetta.

  -----------------------------
  void CheckDelayMessages(hWnd) - onko viivstettyj viestej
  -----------------------------
    Tm funktio sijoitetaan viestisilmukan loppuun ja se tarkistaa onko
    jnyt viivstetysti suoritettavia viestej ksittelemtt.

Vikoja:
    - suunniteltu toistaiseksi vain yhdelle prosessille (ikkunalle)
      kerrallaan.  Eli ainakaan DLL: tst ei saa ilman muutoksia,
      (ikkunan kahva -kohtaiset muuttujat)
    - ongelmia wm_syncpaint-viestin kanssa, eli jos ikkunan ulkopuolella
      siirretn toista ikkunaa piirron aikana, niin piirtv ikkuna
      saa VM_PAINT-viestin, vaikkei sen sislt tuhoudukaan!
      (tt ongelmaa ei ole Windows 3.0:ssa!?)


*****************************************************************************/
//#include <windows.h>
#ifndef NOT_PRINTER
#include "tulostus.h"           /* Jos tulostus halutaan                    */
#endif
#include "checker.h"

#ifdef _TULOSTUS
#define CMChecking  ( CM.Checking || GlobalPrinter.Printing ) /* Tulostus   */
#else
#define CMChecking  ( CM.Checking )
#endif

/****************************************************************************/
/* Tarkistajan globaaleja muuttujia:                                        */
typedef struct {
  int Stop;                       /* Tarkistan halutaan aina ilmoittavan 1  */
  int Close;                      /* Viivstetty ikkunan sulkeminen         */
  int Checking;                   /* Tarkistaja toiminnassa                 */
  int next_big_paint;             /* Seuraava piirto isona!                 */
} CM_type;

static CM_type CM = {0,0,0};

#ifndef NOT_CHECKER
/****************************************************************************/
int CheckMessage(void)
{
  MSG msg;

#ifdef _TULOSTUS
  if ( GlobalPrinter.Printing ) return GlobalPrinter.BreakPrinting;
#endif

  CM.Checking = 1;
  while ( !CM.Stop && PeekMessage(&msg,0,0,0,PM_REMOVE ) ) {
    TranslateMessage(&msg);     /* Tulkitaan virtuaaliset np. koodit       */
    DispatchMessage(&msg);      /* Lhetetn viesti ikkunalle              */
  }
  CM.Checking = 0;

  return CM.Stop;
}
#endif

/****************************************************************************/
void CheckDelayMessages(HWND hwnd)
/* Tarkistetaan viivstytetyt viestit.                                      */
{
  if ( CM.Close ) {            /* Onko viivstetty tuhoaminen???            */
    CM.Close = 0;
    SendMessage(hwnd,WM_DESTROY,0,0); /* viivst. tuhoaminen                */
    return;
  }
  if ( CM.next_big_paint ) {
    InvalidateRect(hwnd,NULL,TRUE);
    CM.next_big_paint = 0;
    return;
  }
  return;
}

/****************************************************************************/
static int DelayClose(HWND hWnd)
/* Tuleeko viivstetty sulkeminen?                                          */
{
  if ( CMChecking )  {            /* Tllin tulee viivstetty close!       */
    SetWindowText(hWnd,"Closing when ready!  Wait....");
    CM.Stop = CM.Close = 1;
    return 1;
  }
  return 0;
}

/****************************************************************************/
#ifndef __WIN32__
void
#else
BOOL
#endif
WINAPI CheckEndPaint(HWND hWnd, const PAINTSTRUCT FAR *ps)
/* Tutkitaan saiko piirt rauhassa hommansa loppuun.                       */
{
  if ( ps == NULL ) { /* Jotta CheckDelayMessage voidaan tehd vaikka olisi */
    CheckDelayMessages(hWnd);       /* vain tmn funktion osoite kytss! */
#ifndef __WIN32__
    return;
#else
    return TRUE;
#endif
  }

  EndPaint(hWnd,ps);
  if ( !CM.Stop )   /* Jos sai rauhassa piirt loppuun                     */
    CM.next_big_paint = 0;
#ifdef __WIN32__
  return TRUE;
#endif
}

/****************************************************************************/
int IsCheckingMessage(HWND hWnd,UINT message,WPARAM wParam,LPARAM lParam)
/*
** Tarkistajan osuus ikkunafunktiosta
**
** Palauttaa:
**   1 jos kutsuneen aliohjelman kuuluu palauttaa OK (eli viesti ksitelty)
**  -1 jos kuuluu kutsua oletusksittelij
**   0 jos viesti tytyy ksitell normaalisti omassa aliohjelmassa
*/
{
#pragma argsused
#ifndef __BORLANDC__
#define SIZE_MINIMIZED SIZEICONIC /* Ainakin  QUICK-C tarvitsee:            */
#endif

  switch (message)
  {
      case WM_PAINT:            /* Viesti: Piirr ikkuna uudelleen          */
         /* Jos viestin tarkistuksen aikana tulee WM_PAINT viesti, niin     */
         /* silloin viestin tarkistukselle on syyt ilmoittaa, ett         */
         /* lopettaa piirtmisen.  Kuitenkin piirtmisen jdess kesken    */
         /* on seuraava piirtminen mahdollisesti vajaa!  Siis parasta      */
         /* piirt koko kuva uudelleen!  Koko kuvan piirto on onnistunut   */
         /* mikli piirtoa ei ole hiritty! (vrt. CheckEndPaint)            */
         if ( CMChecking ) {
           CM.Stop = 1;
           CM.next_big_paint = 1;
           return -1;
         }
         CM.Stop = 0;
         break;

      case WM_SIZE:             /* Jos ikkuna ikoniksi, ei kannata piirt  */
         if ( wParam == SIZE_MINIMIZED ) CM.Stop = 1;
         break;

      case WM_CLOSE:            /* Ikkuna halutaan sulkea!                  */
         /* Jos viestin tarkistuksen aikana tulee WM_CLOSE viesti, pit    */
         /* tarkistuksen pyytnyt ohjelma lopettaa ja pst tekemn       */
         /* sulkeminen pviestisilmukassa.                                 */
         if ( DelayClose(hWnd) ) return 1;
         break;

      case WM_DESTROY:          /* Viesti: ikkuna hvitetn                */
         if ( DelayClose(hWnd) ) return 1;
         break;
  }

  /* Thn loppuu "moniajon" vaatimat muutokset!                            */
  /*------------------------------------------------------------------------*/
  return 0;
}