/**************/
/* mdialog.c  */
/****************************************************************************

    MODULE:  mdialog.c
    PURPOSE: Modeless ja modal dialogien ksittely.  Kirjaston avulla modal
             ja modeless dialogien kytt on yhtenevist, eik esim.
             dialogin funktiossa tarvitse mietti kummastako tyypist on kyse.
    AUTHOR:  Vesa Lappalainen 15.7.1993

    UPADTE:  2.12.1993/vl: DialogSethInastance ja Get
                 - jotta voidaan tehd isttmi dialogeja
             19.8.1994/vl:
                 + DoClass.. ja kaikki ...Param -funktiot
                 + makroja valmiin pohjelman tekemiseksi
                 + m-alkuiset funktiot (mm. mMessageLoop)
             1.10.1994/vl
                 + WC-loppuisia makroja ja nist johtuvia aliohjelmien
                   muutoksia (kuitenkin yhteensopiva edellisen kanssa),
                   jotta luokan tietoihin voi vaikuttaa enemmn
            15.10.1994/vl
                 + mGetChildRect
            22.10.1994/vl
                 + muutoksi WC-loppuisiin fuktioihin ja makroihin
            22.9.1995/vl
                 + WC loppuisia Do...ParamWC dialogin luonteja,
                   joiden avulla voidaan muuttaa dialogin ominaisuuksia
                   ennen dialogin ilmenstymist (esim. tyyli),
                   ks. lokaalin aliohjelman FindUserDialog kommentit

    USAGE:   Kirjaston kyttmiseksi tarvitaan tiedostot:
                 mdialog.c
                 mdialog.h


    Aliohjelmakirjaston kyttmiseksi pit ohjelmoijan tehd ohjelmaansa
    seuraavat muutokset:
      1) Luotava kaikki dialogit jollakin kutsuista
           DoModalDialog(hWnd, name, dialFunc);
           DoModelessDialog(hWnd, name, dialFunc);
           DoMultiModelessDialog(hWnd, name, dialFunc);
           DoModalDialogParam(hWnd, name, dialFunc,lParam);
           DoModelessDialogParam(hWnd, name, dialFunc,lParam);
           DoMultiModelessDialogParam(hWnd, name, dialFunc,lParam);
           DoClassDialogParam(hInstance,classname,iconname,hWnd,name,wndFunc,lParam);
           DoModelessDialogParamWC(hWnd, name, dialFunc,lParam,wcfunc);
           DoMultiModelessDialogParamWC(hWnd, name, dialFunc,lParam,wcfunc);
           DoClassDialogParamWC(hInst...iconname,hWnd,name,wndFunc,lParam,wcfunc);

           Modal         - dialogin aikana muut saman ohjelman ikkunat eivt
                           voi tulla aktiivisiksi
           Modeless      - dialogeja voi olla kynniss 0-1 kerrallaan, muut
                           ikkunat voivat tulla aktiivisiksi

           MultiModeless - dialogeja voi olla 0-n kerrallaan, muut ikkunat
                           voivat tulla aktiivisiksi.  Dialogin funktio tytyy
                           olla totetutettu siten, ettei sen staattiset
                           muutujat hiritse muita saman dialogin esiintymi.
                           Kytnss kaikki tieto voidaan tallettaa dialogin
                           kenttiin ja siirt sielt ennen dialogin
                           sulkemista pohjelmaan jollakin sopivalla
                           aliohjelmakutsulla.
                           Toinen tapa on kytt dialogin lokaalia muistia,
                           ks DialogAlloc jne...
           ClassDialog   - luodaan dialogille luokka ja kynnistetn dialogi
                           MultiModeless-kutsulla
           Param         - vlitetn arvo viestin WM_INITDIALOG lParam:ksi.
           wcfunc        - Dialogin asetuksia muuttvat funktio,
                           ks. lokaalin aliohjelman FindUserDialog kommentit

         Jos halutaan kytt hWnd = NULL, tytyy dialogeille kertoa ohjelman
         esiintymn kahva (pohjelmassa) kutsulla
           DialogSethInstance(hInstance)

      2) Dialogin loppuessa kytettv kutsua:
           DestroyDialog(hDlg, retval);
         tai
           KillDialog(hDlg, retval);

      3) Listtv viestisilmukkaan:
            while (GetMessage(&msg,NULL,NULL,NULL)) {
                if ( IsModelessMessage(&msg) ) continue; // Tm lis!
                ...
                TranslateMessage(&msg);
                DispatchMessage(&msg);
            }
         ( tai kytettv kutsua mMessageLoop )

      4) Listtv ohjelman loppuun:
           FreeAllModeless();


      5) Jos halutaan kytt lokaalia, jokaiselle dialogille omaa
         muistialuetta, pit dialogin funktio kirjoittaa muotoon:

         typedef struct {
            ...
         } *pLocalStruct;

         BOOL CALLBACK _export DialogFunc(...)
         {
           pLocalStruct lp = (pLocalStruct)DialogLock(hDlg);
         ...

           if ( !lp ) {
             lp = (pLocalStruct)DialogAlloc(hDlg,sizeof(lp *))
             if ( !lp ) return FALSE;
             lp->... = alustukset
           }
         ...
           switch (message) {
             ...
             viittauksia lp->...
             ...
           }
           DialogUnlock(hDlg);
           return ret_val;
         }

         Tosin usein poistumisen joutuu tekemn jopa goto-lauseella,
         aliohjelman loppuun, koska pelklle return-lauseella DialogUnlock
         j kunkin case-kohdan jlkeen muuten tekemtt.

      6) About-tyylisten dialogien ksittelyyn on valmis funktio
         nimelt MDialogAbout.  Jos dialogi "About" luodaan kutsulla
           DoModalDialogParam(hWnd,"About",MDialogAbout,(LPARAM)&Version);
         miss
           static tVersionInfo Version = {IDC_VERSION,"Versio 0.2 10.8.1994"};
         niin tm funktio vaihtaa dialogin IDC_VERSION kohdassa olevaksi
         tekstiksi tekstin "Versio 0.2 10.8.1994".
         Nin versionumero voidaa silytt varsinaisessa pohjelmassa
         tarvitsematta aina olla muuttamassa .RC-tiedostoa!
         Tt ominaisuuttahan ei ole pakko kytt ja MDialogAbout toimii
         hyvin pelkll seuraavalla kutsulla
           DoModalDialog(hWnd,"About",MDialogAbout);
         Nin About-dialogien eteen ei tarvitse tehd muuta kuin pelkk
         "About"-niminen dialogi .RC-tiedostoon ja em. kutsu sitten
         sopivassa kohti ohjelmaa.
         MDialogAbout sopii tarvittaessa mink tahansa dialogin
         tilapis-funktioksi, kunnes varsinainen funktio "jaksetaan"
         rakentaa!


============
Aliohjelmat: (mdialog.c, mdialog.h) (M = makro)
============
  HWND DoModalDialog(HWND, char *, PDIALOGPROC);                  (M)
  HWND DoModelessDialog(HWND, char *, PDIALOGPROC);               (M)
  HWND DoMultiModelessDialog(HWND, char *, PDIALOGPROC);          (M)
  HWND DoClassDialog(HINSTANCE,char *,char *,HWND,char *,WPR,DPR);(M)
  HWND DoModalDialogParam(HWND, char *, PDIALOGPROC,LPARAM);
  HWND DoModelessDialogParam(HWND, char *, PDIALOGPROC,LPARAM);
  HWND DoMultiModelessDialogParam(HWND, char *, PDIALOGPROC,LPARAM);
  HWND DoClassDialogParam(HINSTANCE hInstance,
                          char *classname,
                          char *iconname,
                          HWND hWnd,
                          char *dialogname,
                          WNDPROC MainWndProc,
                          DLGPROC MainDlgProc,
                          LPARAM lParam);
  HWND DoClassDialogParamCMD(HINSTANCE hInstance,
                          char *classname,
                          char *iconname,
                          HWND hWnd,
                          char *dialogname,
                          WNDPROC MainWndProc,
                          DLGPROC MainDlgProc,
                          LPARAM lParam,
                          int CmdShow,
                          tWNDCLASSHandler wcHandler)
  int mRegisterWndClass(HINSTANCE hInstance,
                          char *classname,
                          char *iconname,
                          WNDPROC MainWndProc,
                          tWNDCLASSHandler wcHandler)
  void FreeAllModeless(void);
  int DestroyDialog(HWND, int);
  int KillDialog(HWND, int);
  int IsModelessMessage(MSG *);
  void *DialogAlloc(HWND hWnd,UINT size)
  void *DialogLock(HWND hWnd)
  BOOL DialogUnlock(HWND hWnd)
  BOOL DialogFree(HWND hWnd)
  void DialogSethInstance(HINSTANCE hInstance)
  HINSTANCE DialogGethInstance(HWND hWnd)
  BOOL CALLBACK _export MDialogAbout(HWND hDlg,unsigned message,
                            WPARAM wParam,LPARAM lParam);
  LPARAM GetDialogCreateLParam(HWND hWnd);
  int mAddAccelerator(HWND hWnd,char *name);
  int mMessageLoop(void);
  int mIsAcceleratorMessage(MSG *msg);
  int mDeleteAccelerator(HWND hWnd);
  int mGetChildRect(HWND hWnd,int id,RECT *rc)

=============================
Muut makrot:
=============================
  mDlgMAIN(accel,dial,fun)
  mClassDlgMAIN(dclass,icon,accel,dial,fun)


=============================
Aliohjelmien tarkempi kuvaus:
=============================
  ----------------------
  HWND DoModalDialog(HWND, char *, PDIALOGPROC);
  HWND DoModelessDialog(HWND, char *, PDIALOGPROC);
  HWND DoMultiModelessDialog(HWND, char *, PDIALOGPROC);
  HWND DoModalDialogParam(HWND, char *, PDIALOGPROC,LPARAM);
  HWND DoModelessDialogParam(HWND, char *, PDIALOGPROC,LPARAM);
  HWND DoMultiModelessDialogParam(HWND, char *, PDIALOGPROC,LPARAM);
  HWND DoClassDialogParam(HINSTANCE hInstance,
                          char *classname,
                          char *iconname,
                          HWND hWnd,
                          char *dialogname,
                          WNDPROC MainDlgProc,
                          LPARAM lParam);
  HWND DoClassDialogParamCMD(HINSTANCE hInstance,
                          char *classname,
                          char *iconname,
                          HWND hWnd,
                          char *dialogname,
                          WNDPROC MainWndProc,
                          DLGPROC MainDlgProc,
                          LPARAM lParam,
                          int CmdShow,
                          tWNDCLASSHandler wcHandler)
  lisksi samat makroina ilman Param-loppua
  ----------------------
    Nill funktiolla luodaan haluttu dialogi tyyppi.
    Palauttaa:
       NULL  = dialogia ei voitu luoda
       kahva = dialogin kahva
       Modal-tapauksessa dialogin palauttama arvo.
    Parametrit:
       HWND hWnd         = isikkunan kahva (jos NULL, pit olla kytetty
                           kutsua DialogSethInstance)
       char *name        = dialogin nimi
       PDIALOGPROC func  = osoitin dialogin funktioon
       LPARAM lParam     = Param-loppuisten funktioiden WM_INITDIALOG-viestille
                           vietv lParam arvo.
       int CmdShow       = tapa miten ikkuna nytetn (SW_SHOW jos ei muuta
                           keksi) 
       tWNDCLASSHandler wcHandler
                         = ksittelij funktio, joka muuttaa ikkunaluokan
                           ominaisuuksia, esim.
                           void setwc(WNDCLASS *wc) {  wc->style = CS_OWNDC;}

    Modeless-kutsut tallettavat dialogin tiedot sisiseen tietorakenteeseen.
    DoMultiModelessDialog luo aina uuden dialogin esiintymn ja
    DoModelessDialog      ottaa kyttn entisen esiintymn, mikli sellainen
                          on
    DoClass..            = luo dialogin tarvitseman luokan ja synnytt
                           dialogin.  Tm on tarkoitettu niihi tilanteisiin,
                           joissa dialogi esiintyy ohjelman pikkunana.
                           Kutsu voi esiinty useasti samassa pohjemassa.
                           Kutsussa on kaksi funktiota, joista WNDPROC on
                           ikkunaluokan oletusfunktio ja DLGPROC on dialogin
                           funktio, jota kutsutaan, jos oletusfunktio ei
                           ksitellyt viesti.  Esim. WM_CREATE-viesti
                           saadaan vain WNDPROCille ja WM_INITDIALOG vain
                           DLGPROCille.
                           Tmn funktion avulla on tarkoitus pst
                           yksinkertaiseen pohjelmaan, esim. tyyliin:
     --------------------------------------------------------------------------
      int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                         LPSTR lpCmdLine, int nCmdShow)
      {
      #pragma argsused
        HWND hWnd = DoClassDialog(hInstance,"DaDWClass","ikoni",
                                  NULL,"DaDSave",DefDlgProc,MainDlgProc);
        if ( !hWnd ) return 1;
        mAddAccelerator(hWnd,"Pika");
        return mMessageLoop();
      }
     --------------------------------------------------------------------------

    Sama saadaan aikaan mys makrolla mClassDlgMAIN:
      mClassDlgMAIN("DaDWClass","ikoni","Pika","DaDSave",DefDlgProc,MainDlgProc)

    Pohjelma luokattomille dialogeille saadaan kutsulla:
      mDlgMAIN("DaDSave","ikoni",MainDlgProc,"Pika")
    Tmn suurin vika on siin, ettei saada ikonia pikkunalle.

  ----------------------
  int mRegisterWndClass(HINSTANCE hInstance,
                        char *classname,
                        char *iconname,
                        WNDPROC MainWndProc,
                        tWNDCLASSHandler wcHandler)
  ----------------------
    Suorittaa ikkunaluokan rekisterinnin.

  ----------------------
  void FreeAllModeless(void);
  ----------------------
    Tt pit kutsua ohjelma lopuksi (mikli on kytetty yhdenkin kerran
    kutsua DoModelessDialog).  Turhaan kutsuminen ei haittaa!

  ----------------------
  int DestroyDialog(HWND, int);
  ----------------------
    Tll kutsulla poistetaan dialogi, kun sit ei en kytet.
    Jos dialogi luotu kutsulla DoModelessDialog ja se poistetaan
    tll kutsulla, kytetn seuraavan luonnin DoModelessDialog
    yhteydess tt samaa dialogia.  Vain ensimmisen kutsun
    aikana saadaan WM_INITDIALOG -viesti.

    Parametrit:
       HWND hWnd         = poistettavan dialogin kahva
       int  RetVal       = arvo joka halutaan palauttaa kutsuneelle ohjelmalle
                           (merkityst vain Modal-dialogeissa)

  ----------------------
  int KillDialog(HWND, int);
  ----------------------
    Kuten edell, mutta poistetaan dialogi kokonaan, eli uusi DoModelessKutsu
    luo kokonaan uuden dialogin.  Tllin edellisen kerran arvot menetetn
    ja luonnin yhteydess saadaan WM_INITDIALOG -viesti.

  ----------------------
  int IsModelessMessage(MSG *);
  ----------------------
    Tutkitaan kuuluuko viesti jollekin ModelessDialogille.  Tm kutsu
    tytyy list pohjelman viestisilmukkaan.   Jos yhtn dialogia
    ei ole kynniss, kutsu ei aiheuta mitn.
    Palauttaa:
       0     = ei ole minkn dialogin viesti
       1     = on dialogin viesti, viesti ksitelty


  ----------------------
  void *DialogAlloc(HWND hWnd,UINT size)
  ----------------------
    Varataan ja lukitaan tila dialogin omille lokaaleille muuttujille.
    Palautetaan kyttkelpoinen osoite tai NULL-jos eponnistuu.
    Voidaan kytt mys Realloc-tarkoitukseen.

  ----------------------
  void *DialogLock(HWND hWnd)
  ----------------------
    Lukitaan dialogin lokaali muisti.  Jos tilaa ei ole edes varattu
    palautetaan NULL, muuten kyttkelpoinen osoite.

  ----------------------
  BOOL DialogUnlock(HWND hWnd)
  ----------------------
    Poistetaan lukitus dialogin lokaaleilta muuttujilta.  Jos
    halutaan takaisin, pit kutsua DialogLock-funktiota.

  ----------------------
  BOOL DialogFree(HWND hWnd)
  ----------------------
    Vapautetaan dialogin viem lokaali muistitila.  Tm tehdn
    automaattisesti kunkin dialogin hvittmisen yhteydess, joten
    DestroyDialog ja FreeAllModeless -kutsut riittvt.

  ----------------------
  void DialogSethInstance(HINSTANCE hInstance)
  ----------------------
    Laitetaan ohjelman hInstance jemmaan, jos halutaan kutsua
    luonteja NULL-kahvalla.  Kutsutaan yleens pohjelman alussa.

  ----------------------
  HINSTANCE DialogGethInstance(HWND hWnd)
  ----------------------
    Palauttaa hWnd:t vastaavan esiintymn kahvan jos hWnd <> NULL.
    Muuten palauttaa DialogSethInstancella asetetun kahvan (tai NULL).

  ----------------------
  LPARAM GetDialogCreateLParam(HWND hWnd);
  ----------------------
    Jos dialogia on ruvettu luomaan kutsulla Create...Param, niin
    tm palauttaa kutsussa olleen lParam arvon.  Tt kytetn
    oikeasti WM_INITDIALOG-viestiss, mutta jos dialogi on luotu
    luokkana (sille on annettu CLASS), niin WM_INITDIALOGia ei
    kutsuta.  Tll funktiolla voidaan hakea luontiparametri niss
    tapauksissa esim. WM_CRETE-viestiss tai WM_SHOWWINDOW-viestiss.
    (tabhand.h hoitelee tmn automaattisesti, ks. sielt).

  ----------------------
  int mMessageLoop(void)
  ----------------------
    Hoitaa viestisilmukkaa.  Palauttaa msg.wParam arvon.
    Kytetn esim. pohjelmassa:
      return MMessageLoop();

  ----------------------
  int mIsAcceleratorMessage(MSG *msg);
  ----------------------
    Tarkistaa onko viesti jokin pikanppin joka ilmoitettu kutsulla
    MAddAccelerator.

  ----------------------
  int mAddAccelerator(HWND hWnd,char *name);
  ----------------------
    Lis sisiseen taulukkoon pikanppintarkistuksen name
    ikkunalle hWnd.  Tst on hyty jos kytetn MMessageLoop-kutsua,
    tai kytettess kutsua MIsAccelerator (jota MMEssageLoop-kytt).

  ----------------------
  int mDeleteAccelerator(HWND hWnd);
  ----------------------
    Poistetaan ikkunan hWnd pikanppimet sisisest taulukosta.  Tt
    kutsuttava ennen ikkunan tuhoamista.

  ----------------------
  int mGetChildRect(HWND hWnd,int id,RECT *rc)
  ----------------------
    Palauttaa lapsi-ikkunan koordinaatit Client-alueessa.

  ----------------------
  EVENT EVENT_handler_About(tMSGParam *msg)
  ----------------------
    Valmis tabhand.h:n mukainen ksittelijfunktio About-viestiin
    vastaamiseksi
    Tmn lismiseksi viestitaulukkoon on mys makro (ks. tabhand.h)
          EV_HANDLE_WM_COMMAND_CM_HELPABOUT(Version)

****************************************************************************/
#include <windows.h>
#include <windowsx.h>
#include <string.h>
#include "mdialog.h"
#include "tabhand.h"

#define MODE_MODAL           2
#define MODE_MODELESS        0
#define MODE_MULTI_MODELESS  1

/* Seuraava muuttuja on ...CreateParam kutsujen parametrin silyttmiseksi */
/* Se tytyy jossakin vaiheessa korvata taulukolla, jossa on jemmassa      */
/* usamman dialogin luontiin liittyvt lParam:it.  Tm toimii vain jos    */
/* vain yht dialogia ollaan luomassa kerrallaan.                          */
static LPARAM GlobalLParam = 0;

typedef struct { /* Tyyppi, johon talletetaan voimassa olevat modeless-dial*/
  char name[20];      /* Dialogin-nimi                                     */
  int visible;        /* 1 = on nkyviss                                  */
  HWND hwnd;          /* Dialogin kahva                                    */
  FARPROC lpProcFunc; /* Dialogin funktion osoite                          */
  int mode;           /* 0 = vain 1 esiintym, 1 = useita esiintymi       */
                      /* 2 = modal dialog                                  */

  HLOCAL hloc;        /* Varatun muistin kahva                             */
  int    size;        /* Varatun tilan koko                                */
} modeless_type;

#define MAX_DIALOGS 10
static struct {
  int n;              /* Dialogien lukumr                               */
  HINSTANCE hInstance;/* Esiintymn kahva                                  */
  modeless_type dialogs[MAX_DIALOGS];
} Modeless = {0,NULL};

/***************************************************************************/
void DialogSethInstance(HINSTANCE hInstance)
{
  if ( hInstance == NULL ) return;
  Modeless.hInstance = hInstance;
}

/***************************************************************************/
HINSTANCE DialogGethInstance(HWND hWnd)
{
  if ( hWnd ) return GetWindowInstance(hWnd);
  return Modeless.hInstance;
}

/***************************************************************************/
static void mFreeProcInstance(FARPROC lpProcFunc)
{
  if ( lpProcFunc == NULL ) return;
  FreeProcInstance(lpProcFunc);
}

/***************************************************************************/
static int FindModelessIndex(HWND hWnd)
{
  int i;
  for (i=0; i<Modeless.n; i++)
    if ( Modeless.dialogs[i].hwnd == hWnd ) return i;

  if ( Modeless.dialogs[--i].hwnd == 0) {  /* Jos luonti menossa.         */
    Modeless.dialogs[i].hwnd = hWnd;       /* Niin arvoksi tm kahva     */
    return i;
  }

  return -1;
}

/***************************************************************************/
static modeless_type *FindModeless(HWND hWnd)
{
  int i = FindModelessIndex(hWnd);
  if ( i < 0 ) return NULL;
  return &Modeless.dialogs[i];
}

/***************************************************************************/
static modeless_type *FindModelessName(const char *name)
/* Etsitn dialogi, jonka nimi  on name ja mode = MODALESS                */
{
  int i;
  modeless_type *m;

  for (i=0; i<Modeless.n; i++) {
    m = &Modeless.dialogs[i];
    if ( strcmp(m->name,name) == 0  && m->mode == MODE_MODELESS)  
      return m;
  }

  return NULL;
}


/***************************************************************************/
static BOOL FreeModelessMem(modeless_type *m)
/* Vapautetaan mahdollinen lokaalin alueen muisti!                         */
{
  UINT nlocs;
  unsigned int i;
  if ( !m || !m->hloc ) return FALSE;

  nlocs = LocalFlags(m->hloc) & 0xff;
  for (i=1; i<=nlocs; i++) LocalUnlock(m->hloc);
  if ( LocalFree(m->hloc) == NULL ) { /* Jos vapautus onnistui             */
    m->hloc = NULL;
    m->size = 0;
    return TRUE;
  }

  return FALSE;
}


/***************************************************************************/
static void RemoveModelessIndex(int index, int retval)
/* Poistaa i:en modeless-dialogin kokonaan joukosta                        */
{
  int i;
  modeless_type *m = &Modeless.dialogs[index];


  if ( m->mode != MODE_MODAL )
    DestroyWindow(m->hwnd);
  else
    EndDialog(m->hwnd,retval);

  mFreeProcInstance(m->lpProcFunc);

  FreeModelessMem(m);

  for (i=index; i<Modeless.n-1; i++)
    Modeless.dialogs[i] = Modeless.dialogs[i+1];
  Modeless.n--;

}


/***************************************************************************/
int IsModelessMessage(MSG *msg)
{
  int i;
  for (i=0; i<Modeless.n; i++)
    if ( Modeless.dialogs[i].visible &&
         IsDialogMessage(Modeless.dialogs[i].hwnd,msg) ) return 1;
  return 0;
}

/***************************************************************************/
static tDlgInfo FindUserDialog(HINSTANCE hInstance, const char *name,
                               tWNDCLASSHandler wcHandler)
/* Dialogi luodaan CreateDialogIndirectParam kutsulla, jotta kyttj
   psee tarvittaessa muuttamaan dialogin luontiparametreja.
   Kyttjn aliohjelmaan wcHandler vlitetn parametrina tietue, jossa
   on valmiina dialogi-infon
     hgl        globaali kahva ja
     Tmpl       lukittu muistiosoite.
     Del  = 1   tytyyk muistialue tuhota
   Jos kyttj haluaa vaihtaa nm, niin hn voi sen tehd, mutta
   silloin hnen on poistettava lukitus ja tuhottava vanha alue kutsulla
     DeleteDialogInfo(c->Dlg);
   Omat arvonsa kyttj voi laittaa sitten lukittuun osoitteeseen
   ja jos hn haluaa muistialueen tuhoutuvan dialogin luomisen jlkeen,
   on  palautettava Del=1.
   Eli kyttjn on aluetta muuttaessaan palautettava sek
   kahva ett lukittu muistiosoite tai lukitsematon kahva ja
   NULL.

   Jos kyttj haluaa pelkstn muuttaa dialogin tietoja, niin
   hn voi sen tehd kyttmll suoraan dialogin tietojen kahvaa.
   Kyttjn funktiota wcHandler voidaan kutsua siten ett tietueen
   tCreWndSt kentt Dlg == NULL, jolloin ei haluta dialogin tietoja,
   vaan ikkunaluokan luomiseen liittyvy arvoja.

   Esimerkiksi dialogi muutetaan systemmodaaliksi seuraavalla ksittelyll

     static void setwc(tCreWndSt *c) {
       if ( c->Dlg == NULL ) return;
       FindIniFileName(N_S(G.ini),NULL);
       if ( c->Dlg->Tmpl &&
           GetPrivateProfileInt(WinLoginPOS,"sysmodal",0,G.ini) ) {
         DWORD far *style = (DWORD far *)(c->Dlg->Tmpl);
         *style = *style | DS_SYSMODAL;
       }
     }

-----------------------------------------------------------------------------*/
{
  tCreWndSt c;
  tDlgInfo  Dlg;
  HRSRC hres = FindResource(hInstance,name,RT_DIALOG);
  c.Dlg = &Dlg;
  Dlg.hgl =  LoadResource(hInstance,hres);
  Dlg.Tmpl = LockResource(Dlg.hgl);
  Dlg.Del  = 1;
  if ( wcHandler ) wcHandler(&c);
  if ( Dlg.hgl && Dlg.Tmpl == NULL ) Dlg.Tmpl = LockResource(Dlg.hgl);
  return Dlg;
}

/***************************************************************************/
void UnlockDialogInfo(tDlgInfo *Dlg)
{
  if ( Dlg && Dlg->Tmpl ) {
    (void)UnlockResource(Dlg->hgl);
    Dlg->Tmpl = NULL;
  }
}

/***************************************************************************/
void DeleteDialogInfo(tDlgInfo *Dlg)
{
  UnlockDialogInfo(Dlg);
  if ( Dlg && Dlg->Del ) {
    if ( Dlg->hgl) FreeResource(Dlg->hgl);
    Dlg->hgl  = NULL;
    Dlg->Del  = 0;
  }
}

/***************************************************************************/
static HWND DoNewModelessWC(HWND hWnd,const char *name,PDIALOGPROC func,
                            int mode,LPARAM lParam,
                            tWNDCLASSHandler wcHandler)
/* Luodaan uusi Modeless Dialog-ikkuna                                     */
/* Jos kutsutaan Modal-dialogille, pit kahva olla valmiina hWnd:ss ja   */
/* mode = MODE_MODAL                                                       */
/* Koska CreateDialog kutsuu dialogin funktiota alustusviestill, ja tm  */
/* saattaa jo pyyt luomaan DialogAllocilla muistia, pit tss olla     */
/* varovainen ja pit huoli, ett dialogin tiedot ovat valmiina jo ennen  */
/* kuin CreateDialogista on palattu.  Muut tiedothan saadaankin kohdalleen */
/* Mutta dialogin kahva j tuntemattomaksi.  Siksi se merkitnkin        */
/* arvolla 0,  joka muissa aliohjelmissa (FindModelessIndex)               */
/* tulkitaan keskeneriseksi luonniksi.                                    */
{
  HWND nhWnd;
  HINSTANCE hInstance = DialogGethInstance(hWnd);
  modeless_type *m;

  if ( Modeless.n >= MAX_DIALOGS ) return 0;

  m = &Modeless.dialogs[Modeless.n];
  Modeless.n++;
  strncpy(m->name,name,sizeof(m->name));
  m->visible    = 1;
  m->mode       = mode;
  m->lpProcFunc = NULL;
  m->hloc       = NULL;
  m->size       = 0;
  m->hwnd       = 0;       /* Luonti menossa!                              */

  if ( mode == MODE_MODAL ) m->hwnd = hWnd; /* kahva pit olla jo valmiina*/
  else { /* Ei modal                                                       */
    tDlgInfo Dlg = FindUserDialog(hInstance,name,wcHandler);
    m->lpProcFunc = MakeProcInstance((FARPROC)func, hInstance);
    GlobalLParam = lParam;
    nhWnd = NULL;
    if ( Dlg.Tmpl )
      nhWnd = CreateDialogIndirectParam(hInstance, Dlg.Tmpl, hWnd,
                                       (DLGPROC)m->lpProcFunc,lParam);
    DeleteDialogInfo(&Dlg);
#if 0
    m->lpProcFunc = MakeProcInstance((FARPROC)func, hInstance);
    GlobalLParam = lParam;
    nhWnd = CreateDialogParam(hInstance, name, hWnd, (DLGPROC)m->lpProcFunc,lParam);
#endif
    /* CreateDialog kutsuu dialogin funktiota, ja tm saattaa tarvita     */
    /* lokaalia muistia, jolloin m->hwnd saatetaan saada selville jo       */
    /* ennen CreateDialogista paluuta eik se siis paluun jlkeen ole 0    */
    /* Mutta mitp jos luonteja on ollut useita yhtaikaa, mist tunnistaa*/
    /* mille luonnille mikkin muistialue kuuluu???                        */
    /* Ongelmia ei pitisi tulla, mikli DialogAlloc-kutsu on heti         */
    /* dialogin alustusviestin ksittelyn alussa tai jopa ennen!           */

    if ( m->hwnd && m->hwnd != nhWnd ) { /* Tt ei koskaan testattu/vl-93 */
      MessageBox(hWnd,"Useita luonteja yhtaikaa","Vikaa!!!",MB_OK);
      nhWnd = NULL;
    }
    m->hwnd = nhWnd;

  }

  if ( !m->hwnd )
    RemoveModelessIndex(Modeless.n-1,1);

  return m->hwnd;
}


/***************************************************************************/
HWND DoModelessDialogParamWC(HWND hWnd,const char *name,PDIALOGPROC func,
                             LPARAM lParam,tWNDCLASSHandler wcHandler)
/* Luodaan uusi Modeless Dialog-ikkuna                                     */
/* Mikli ennestn on olemassa vastaava ikkuna, niin kytetn sen        */
/* valmiiksitehty funktiota.                                              */
/* Mikli dialogi on ennestn, ei luoda uutta esiintym, vaan laitetaan  */
/* vanha aktiiviseksi.                                                     */
/* Palautetaan 0, mikli ikkunaa ei voi luoda, muuten kahva ikkunaan       */
{
  modeless_type *m;

  m = FindModelessName(name);
  if ( !m ) return DoNewModelessWC(hWnd,name,func,MODE_MODELESS,lParam,wcHandler);

  if ( m->visible == 0 )           /* Jollei nkyviss, niin nkyviin      */
    ShowWindow(m->hwnd,SW_SHOWNORMAL);
  m->visible=1;
  SetFocus(m->hwnd);               /* Vaihdetaan focus aina jos on ik.     */

  return m->hwnd;
}


/***************************************************************************/
HWND DoMultiModelessDialogParamWC(HWND hWnd,const char *name,PDIALOGPROC func,
                                LPARAM lParam,tWNDCLASSHandler wcHandler)
/* Luodaan uusi Modeless Dialog-ikkuna                                     */
/* Sallitaan saman esiinty useita kertoja.                                */
{
  return DoNewModelessWC(hWnd,name,func,MODE_MULTI_MODELESS,lParam,wcHandler);
}


/***************************************************************************/
void FreeAllModeless(void)
{
  int i;
  for (i=Modeless.n-1; i>=0; i--)
    RemoveModelessIndex(i,1);
}


/***************************************************************************/
void RemoveModeless(HWND hWnd,int retval)
/* Poistaa i:en modeless-dialogin kokonaan joukosta                        */
{
  int index = FindModelessIndex(hWnd);
  RemoveModelessIndex(index,retval);
}


/***************************************************************************/
int DestroyDialog(HWND hWnd,int retval)
/* Tuhoaa modeless dialogit DestroyWindow-komennolla ja muut dialogit      */
/* EndDialog -komennolla (Modal)                                           */
/* Jos mode = 1, jtetn dialogi olemaan.                                 */
{
  modeless_type *m = FindModeless(hWnd);
  if ( !m ) {
    EndDialog(hWnd,retval);
    return retval;
  }

  m->visible = 0;
  if ( m->mode != 0 ) RemoveModeless(hWnd,retval);
  else ShowWindow(m->hwnd,SW_HIDE);
  return retval;
}


/***************************************************************************/
int KillDialog(HWND hWnd,int retval)
/* Tuhoaa modeless dialogit DestroyWindow-komennolla ja muut dialogit      */
/* EndDialog -komennolla (Modal)                                           */
{
  modeless_type *m = FindModeless(hWnd);
  mDeleteAccelerator(hWnd);
  if ( !m ) {
    EndDialog(hWnd,retval);
    return retval;
  }

  RemoveModeless(hWnd,retval);
  return retval;
}

/***************************************************************************/
/* Modeless aliohjelmat loppuivat                                          */
/***************************************************************************/


/***************************************************************************/
HWND DoModalDialogParamWC(HWND hWnd,const char *name,PDIALOGPROC func,
                          LPARAM lParam,tWNDCLASSHandler wcHandler)
{
  int ret = 0;
  HINSTANCE hInstance = DialogGethInstance(hWnd);
  FARPROC lpProcFunc;
  tDlgInfo Dlg = FindUserDialog(hInstance,name,wcHandler);

  lpProcFunc = MakeProcInstance((FARPROC)func, hInstance);
  GlobalLParam = lParam;
//  ret = DialogBoxParam(hInstance, name, hWnd, (DLGPROC)lpProcFunc,lParam);
  if ( Dlg.hgl ) {
    UnlockDialogInfo(&Dlg);
    ret = DialogBoxIndirectParam(hInstance, Dlg.hgl, hWnd,
                               (DLGPROC)lpProcFunc,lParam);
  }
  DeleteDialogInfo(&Dlg);
  mFreeProcInstance(lpProcFunc);
  return (HWND)ret;
}

/***************************************************************************/
LPARAM GetDialogCreateLParam(HWND hWnd)
{
#pragma argsused
  LONG lp = GlobalLParam;
  GlobalLParam = 0;
  return lp;
}   

/***************************************************************************/
/* Lokaalin muistin aliohjelmat alkavat                                    */
/***************************************************************************/

/***************************************************************************/
void *DialogLock(HWND hWnd)
/* Palautetaan lukittu muistialue hWnd:n lokaaleille muuttujille.          */
/* Jos hWnd:t ei ole rekisterity TAI muistia ei ole varattu, palautetaan */
/* NULL                                                                    */
{
  modeless_type *m = FindModeless(hWnd);
  if ( !m ) return NULL;

  if ( !m->hloc ) return NULL;

  return LocalLock(m->hloc);
}


/***************************************************************************/
void *DialogAlloc(HWND hWnd,UINT size)
/* Allokoidaan muistia ja palautetaan muistille lukittu kahva.             */
/* Jos jo allokoitu tlle dialogille, palautetaan vanha kahva.             */
/* Jos eponnistuu, palautetaan NULL                                       */
/* Jos dialogi ei ole listassa, listn se sinne Modal-dialogina          */
/* Tt voidaan kytt mys ReAllocina, kunhan vaan kaikki lukot on pois  */
{
  modeless_type *m = FindModeless(hWnd);

  if ( !m ) {
    DoNewModelessWC(hWnd,"",NULL,MODE_MODAL,0,NULL);
    m = FindModeless(hWnd);
    if ( !m ) return NULL;
  }

  if ( !m->hloc ) { /* Muistia ei viel ole                                */
    m->hloc = LocalAlloc(LMEM_MOVEABLE | LMEM_ZEROINIT,size);
    m->size = size;
  }
  else if ( size != (UINT)(m->size) ) { /* Jos kokoa tarvitsee muuttaa!            */
    if ( LocalFlags(m->hloc) & 0xff ) return NULL; /* Kokoa ei voi muutt.  */
    m->hloc = LocalReAlloc(m->hloc,size,LMEM_MOVEABLE);
    m->size = size;
  }

  if ( !m->hloc ) m->size = 0;

  return DialogLock(hWnd);
}


/***************************************************************************/
BOOL DialogUnlock(HWND hWnd)
/* Poistetaan lukitus dialogin muistista.  Muistin hvitys tulee           */
/* samalla kuin dialogin tuhoaminenkin.                                    */
{
  modeless_type *m = FindModeless(hWnd);
  if ( !m ) return FALSE;

  if ( !m->hloc ) return FALSE;

  return LocalUnlock(m->hloc);
}


/***************************************************************************/
BOOL DialogFree(HWND hWnd)
/* Vapautetaan dialogin lokaali muisti                                     */
{
  return FreeModelessMem(FindModeless(hWnd));
}

/***************************************************************************/
BOOL CALLBACK _export MDialogAbout(HWND hDlg,UINT message,
                                   WPARAM wParam,LPARAM lParam)
{
#pragma argsused
  switch (message) {
    case WM_INITDIALOG: {
      tVersionInfo *v = (tVersionInfo *)lParam;
      if ( v == NULL ) return TRUE;
      SetWindowText(GetDlgItem(hDlg,v->id),v->version);
      return TRUE;
    }
    case WM_COMMAND: {
      int id = GET_WM_COMMAND_ID(wParam,lParam);
      if ( id == IDOK || id == IDCANCEL ) {
        DestroyDialog(hDlg, TRUE);
        return (TRUE);
      }
      break;
    }
  }
  return FALSE;
}


/***************************************************************************/
/* Pikanppinten ksittely                                                */
/***************************************************************************/
typedef struct {
  HWND   hWnd;
  HACCEL hAccel;
  int    loose;
} tOneAccel;

#define MAX_ACCEL 30
typedef struct {
  int n;
  tOneAccel a[MAX_ACCEL];
} tAccels;

static tAccels Accels = {0};

/***************************************************************************/
int mDeleteAccelerator(HWND hWnd)
/* Poistaa pikanppintaulukon tarkistettavien joukkosta                   */
{
  int w,r;
  for (r=0,w=0; r<Accels.n; r++)
    if ( Accels.a[r].hWnd != hWnd )
      Accels.a[w++] = Accels.a[r];
  Accels.n = w;
  return 0;
}

/****************************************************************************/
int mChangeAcceleratorType(HWND hWnd,int loose)
/* Muutetaan pikanppimen tyyppi.  Palautetaan vanha.                      */
{
  int i;
  int ot;
  for (i=0; i<Accels.n; i++)
    if ( Accels.a[i].hWnd != hWnd ) {
      ot = Accels.a[i].loose;
      if ( loose == 0 || loose == 1 ) Accels.a[i].loose = loose;
      return ot;
    }
  return -1;
}

/****************************************************************************/
int mAddAcceleratorType(HWND hWnd,const char *name,int loose)
/* Lis pikanppintaulukon tarkistettavien joukkoon                       */
/* loose = 1 => ei tarvitse olla omassa ikkunassa, loose = 0 => tytyy olla */
{
  HACCEL hAccel;
  int i;
  if ( name == NULL || name[0] == 0 ) return -1;
  if ( Accels.n >= MAX_ACCEL ) return -2;
  hAccel = LoadAccelerators(GetWindowInstance(hWnd),name);
  if ( hAccel == NULL ) return -3;
  i = Accels.n++;
  Accels.a[i].hWnd = hWnd;
  Accels.a[i].hAccel = hAccel;
  Accels.a[i].loose = loose;
  return 0;
}


/****************************************************************************/
int mIsAcceleratorMessage(MSG *msg)
/* Tarkistaa onko jokin kytss oleva pikanppinviesti                    */
{
  int i;
  HWND hWnd = GetActiveWindow(); (void)hWnd;
  for (i=0; i<Accels.n; i++)
    if ( ( Accels.a[i].loose || Accels.a[i].hWnd == hWnd ) &&
         TranslateAccelerator(Accels.a[i].hWnd,Accels.a[i].hAccel,msg) ) {
      return 1;
    }
  return 0;
}

/****************************************************************************/
int mMessageLoop(void)
/* Suorittaa viestisilmukkaa.                                               */
{
  MSG msg;
  while (GetMessage(&msg,0,0,0)) {
    if ( mIsAcceleratorMessage(&msg) ) continue;
    if ( IsModelessMessage(&msg) ) continue;
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  FreeAllModeless();
  return msg.wParam;
}

/****************************************************************************/
int mIsClass(const char *name)
/* Tutkitaan onko luokkaa valmiiksi olemassa vai ei                         */
/* Ei toimi viel!                                                          */
{
  WNDCLASS  wc;   /* Ikkunaluokka    */
  return GetClassInfo(NULL,name,&wc);
}

/****************************************************************************/
BOOL CALLBACK _export mDialogNoClass(HWND hDlg,UINT message,
                                   WPARAM wParam,LPARAM lParam)
{
#pragma argsused
  switch (message) {
    case WM_INITDIALOG: {
      MessageBox(hDlg,"Dialogilla ei ole luokkaa!","MDIALOG ERROR!",MB_ICONHAND);
      return TRUE;
    }
    case WM_COMMAND: {
      int id = GET_WM_COMMAND_ID(wParam,lParam);
      if ( id == IDOK || id == IDCANCEL ) {
        PostQuitMessage(0);
        return TRUE;
      }
      break;
    }
  }
  return FALSE;
}

/****************************************************************************/
int mRegisterWndClass(HINSTANCE hInstance,
                      const char *classname,
                      const char *iconname,
                      WNDPROC MainWndProc,
                      tWNDCLASSHandler wcHandler)
{
  tCreWndSt c;   /* Ikkunaluokka    */

  if ( mIsClass(classname) != 0 ) return 0;
  c.wc.style         = 0;
  c.wc.lpfnWndProc   = MainWndProc;
  c.wc.cbClsExtra    = 0;
  c.wc.cbWndExtra    = DLGWINDOWEXTRA;
  c.wc.hInstance     = hInstance;
  c.wc.hIcon         = iconname ? LoadIcon(hInstance, iconname) :
                                LoadIcon(NULL, IDI_APPLICATION);
  c.wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  c.wc.hbrBackground = (HBRUSH)GetStockObject(GRAY_BRUSH);
  c.wc.lpszMenuName  = NULL;
  c.wc.lpszClassName = classname;
  c.Dlg = NULL;
  if ( wcHandler ) wcHandler(&c);
//    if (!RegisterClass(&wc)) return NULL;
  RegisterClass(&c.wc);

  return 0;
}

/****************************************************************************/
HWND DoClassDialogParamCMD(HINSTANCE hInstance,
                        const char *classname,
                        const char *iconname,
                        HWND hWnd,
                        const char *dialogname,
                        WNDPROC MainWndProc,
                        DLGPROC MainDlgProc,
                        LPARAM lParam,
                        int nCmdShow,
                        tWNDCLASSHandler wcHandler)
{
  HWND    dhWnd;  /* Dialogin kahva  */

  DialogSethInstance(hInstance);

  mRegisterWndClass(DialogGethInstance(hWnd),classname,iconname,
                    MainWndProc,wcHandler);

  dhWnd = DoMultiModelessDialogParamWC(hWnd,dialogname,MainDlgProc,lParam,wcHandler);
  ShowWindow(dhWnd,nCmdShow);
  return dhWnd;
}

/****************************************************************************/
int mGetChildRect(HWND hWnd,int id,RECT *rc)
{
  WINDOWPLACEMENT wndpl;
  HWND hWndChild = GetDlgItem(hWnd,id);
  rc->top = 0; rc->bottom = 0; rc->left = 0; rc->right = 0;
  if ( hWndChild == NULL ) return -1;
  wndpl.length = sizeof(wndpl);
  GetWindowPlacement(hWndChild,&wndpl);
  *rc = wndpl.rcNormalPosition;
  return 0;
}

/****************************************************************************/
EVENT EVENT_handler_About(tMSGParam *msg)
{
  return (int)DoModalDialogParam(msg->hWnd,"About",MDialogAbout,msg->extra);
}

#if 0
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/* 1995 */
//#include <winuser.h>
/***************************************************************************/
static HWND DoNewModelessSet(HWND hWnd,const char *name,PDIALOGPROC func, int mode,LPARAM lParam)
/* Luodaan uusi Modeless Dialog-ikkuna                                     */
/* Jos kutsutaan Modal-dialogille, pit kahva olla valmiina hWnd:ss ja   */
/* mode = MODE_MODAL                                                       */
/* Koska CreateDialog kutsuu dialogin funktiota alustusviestill, ja tm  */
/* saattaa jo pyyt luomaan DialogAllocilla muistia, pit tss olla     */
/* varovainen ja pit huoli, ett dialogin tiedot ovat valmiina jo ennen  */
/* kuin CreateDialogista on palattu.  Muut tiedothan saadaankin kohdalleen */
/* Mutta dialogin kahva j tuntemattomaksi.  Siksi se merkitnkin        */
/* arvolla 0,  joka muissa aliohjelmissa (FindModelessIndex)               */
/* tulkitaan keskeneriseksi luonniksi.                                    */
{
  HWND nhWnd;
  HINSTANCE hInstance = DialogGethInstance(hWnd);
  modeless_type *m;

  if ( Modeless.n >= MAX_DIALOGS ) return 0;

  m = &Modeless.dialogs[Modeless.n];
  Modeless.n++;
  strncpy(m->name,name,sizeof(m->name));
  m->visible    = 1;
  m->mode       = mode;
  m->lpProcFunc = NULL;
  m->hloc       = NULL;
  m->size       = 0;
  m->hwnd       = 0;       /* Luonti menossa!                              */

  if ( mode == MODE_MODAL ) m->hwnd = hWnd; /* kahva pit olla jo valmiina*/
  else { /* Ei modal                                                   */
    HRSRC hres = FindResource(hInstance,name,RT_DIALOG);
    HGLOBAL hglDlg = LoadResource(hInstance,hres);
    void FAR *dlgtmpl = LockResource(hglDlg);
    DWORD *style = (DWORD *)dlgtmpl;
    *style = *style | DS_SYSMODAL;
    m->lpProcFunc = MakeProcInstance((FARPROC)func, hInstance);
    GlobalLParam = lParam;
    nhWnd = CreateDialogIndirectParam(hInstance, dlgtmpl, hWnd, (DLGPROC)m->lpProcFunc,lParam);
    UnlockResource(hglDlg);
    FreeResource(hglDlg);
    /* CreateDialog kutsuu dialogin funktiota, ja tm saattaa tarvita     */
    /* lokaalia muistia, jolloin m->hwnd saatetaan saada selville jo       */
    /* ennen CreateDialogista paluuta eik se siis paluun jlkeen ole 0    */
    /* Mutta mitp jos luonteja on ollut useita yhtaikaa, mist tunnistaa*/
    /* mille luonnille mikkin muistialue kuuluu???                        */
    /* Ongelmia ei pitisi tulla, mikli DialogAlloc-kutsu on heti         */
    /* dialogin alustusviestin ksittelyn alussa tai jopa ennen!           */

    if ( m->hwnd && m->hwnd != nhWnd ) { /* Tt ei koskaan testattu/vl-93 */
      MessageBox(hWnd,"Useita luonteja yhtaikaa","Vikaa!!!",MB_OK);
      nhWnd = NULL;
    }
    m->hwnd = nhWnd;

  }

  if ( !m->hwnd )
    RemoveModelessIndex(Modeless.n-1,1);

  return m->hwnd;
}

/***************************************************************************/
HWND DoMultiModelessDialogParamSet(HWND hWnd,const char *name,PDIALOGPROC func,LPARAM lParam)
/* Luodaan uusi Modeless Dialog-ikkuna                                     */
/* Sallitaan saman esiinty useita kertoja.                                */
{
  return DoNewModelessSet(hWnd,name,func,MODE_MULTI_MODELESS,lParam);
}

/****************************************************************************/
HWND DoClassDialogParamCMDSet(HINSTANCE hInstance,
                        const char *classname,
                        const char *iconname,
                        HWND hWnd,
                        const char *dialogname,
                        WNDPROC MainWndProc,
                        DLGPROC MainDlgProc,
                        LPARAM lParam,
                        int nCmdShow,
                        tWNDCLASSHandler wcHandler)
{
  HWND    dhWnd;  /* Dialogin kahva  */

  DialogSethInstance(hInstance);

  mRegisterWndClass(DialogGethInstance(hWnd),classname,iconname,
                    MainWndProc,wcHandler);

  dhWnd = DoMultiModelessDialogParamSet(hWnd,dialogname,MainDlgProc,lParam);
  ShowWindow(dhWnd,nCmdShow);
  return dhWnd;
}

#endif
