/**************/
/* savepos.c  */
/*****************************************************************************
  PROGRAM: savepos.c
  PURPOSE: Tallettaa tai ottaa ikkunan paikan INI-tiedostosta
  AUTHOR:  Vesa Lappalainen 27.10.1993
  USAGE:   Lukee ja kirjoittaa parametrina tuotavaan INI-tiedostoon
           ikkunan paikan.  INI-tiedoston muoto:
             [Paikka]
             x=10
             y=20
             leveys=200         (vaikuttaa vain jos kutsun size != 0)
             korkeus=120        (vaikuttaa vain jos kutsun size != 0)
             minmax=1           1 = normaali,  2 = ikoni,  3 = maksimoitu

           Aliohjelmaa

             SavePos(HWND hWnd,LPCSTR inifilename,LPCSTR section,int size)

           kutsutaan jossakin ennen ikkunan ikkunan hvimist,
           esim WM_DESTROY tai WM_CLOSE -viestiss.

           Aliohjelmaa

             RestorePos(HWND hWnd,LPCSTR inifilename,LPCSTR section,int size)

           kutsutaan esim. WM_CREATE tai WM_INITDIALOG -viestiss tai
           ikkunan luonnin jlkeen.  Jos kyseess Dialogi, niin
           WM_CREATE vaiheessa ei ole viisasta kutsua tt, vaan
           pit kutsua myhemmin.  Tosin jos dialogilla on luokka,
           niin WM_INITDIALOG-viesti ei saada ikkunafunktioon,
           ja WM_CREATE on liian aikaisin.  Tllin tytyy tehd
           dialogin funktio, jossa WM_INITDIALOG-ksitelln tai
           sitten esim. WM_SHOWWINDOW kutsuun lippu, jolla 1. kutsukerralla
           kutsutaan RestorePos.
           Parametrilla size mrtn vlitetnk ikkunan koosta lainkaan!
           (Esim. dialogeille ei vlttmtt kannata kytt size==1, koska
           jos muutetaan WIndowsin asetuksia, niin dialogin koko muuttuu,
           mutta sit ei voida hiirell palauttaa oikeaan kokoonsa!).
           On mys vakiot SAVEPOS_SIZE (1) ja SAVEPOS_NOSIZE (0).

           Jos inifilaname == NULL, kytetn nimen ohjelman nime+.INI

------------------------------------------------------------------------------
  Samalla aliohjelma on malli seuraavista:
   1.  Miten sama tiedosto voi olla sek "tavallisena"-aliohjelmana
       ett DLL-kirjaston aliohjelmana (makro WINAPI_LIB hoitaa tmn)
       Trke on tllin muistaa vlitt kaikki osoitteet pitkin!
       Siis: ei char *, vaan LPSTR (tai LPCSTR)
             ei RECT *, vaan RECT FAR *

       Osoitteiden tytyy olla pitki, koska jos aliohjelma on DLL:ss,
       on sille vietvt parametrit aivan eri data-segmentiss kuin
       itse kutsuva ohjelma.
       Tt ongelmaahan ei ole jos tehdn "tavallinen" linkitys.

   2.  Miten makrojen avulla voidaan tehd eri tilanteissa eri tavalla
       toimivia "aliohjelmia".
          Makro INI_WNDL joko lukee tai kirjoittaa INI-tiedostoa sen
          mukaan millainen makro INI sattuu olemaan makron INI_WNDL
          kutsuhetkell.

       Tm tilanne tulee usein vastaan nimenomaan tiedoston lukemisen
       ja kirjoittamisen yhteydess, jolloin koodi on itse lukemista
       tai kirjoittamista lukuunottamatta tysin samanlainen.
       Tllaisella makrolla (tai sopivalla aliohjelmalla) saadaan tiedoston
       alkioiden lukumr "piilotettua" vain yhteen paikkaan.

       Tllaisten rakenteiden kytt on kuitenkin vhn "vaarallista", koska
       pit tarkkaan mietti kelpaako sama homma todellakin sek lukemiseen
       ett kirjoittamiseen.  Usein onkin parempi kirjoittaa ensin erilliset
       lukemiset ja kirjoittamiset ja tmn jlkeen koota tarvittaessa
       yhteiset osat samaan aliohjelmaan tai makroon.

  Tehtvi:  1) Kirjoita kaikki tiedoston makrot auki niinkuin ne oikeasti
                tulevat esiintymn (huomaa makrojen siskkisyys).

*****************************************************************************/
#include <windowsx.h>
#include <string.h>
#include "savepos.h"
#include "mjonot.h"

/****************************************************************************/
int WINAPI_LIB ForceInsideScreen(RECT FAR *rc)
/* Pakotetaan ikkuna, jonka koordinaatit ovat rc:ssa, nkymn ainakin
** osittain ruudun alueella siten, ett siihen ainakin pstn ksiksi,
** eli joko oikea tai vasen ylkulma on ruudussa.  Tllin ikkuna saadaan
** ainakin aktivoiduksi ja tmn jlkeen voidaan painaa ALT-SPACE ja saadaan
** liikutus nppimistlt kyntiin.
** Palautetaan montaako koordinaattia jouduttiin muuttamaan. (0= oli sisll)
*/
{
#define INSIDE 10 /* Paljonko ikkunaa tuodaan sisnpin                    */
  RECT sc;
  int width=rc->right-rc->left, height=rc->bottom-rc->top;
  int i = 0;

  GetWindowRect(GetDesktopWindow(),&sc);

  if ( rc->left   >= sc.right  ) { i++; rc->left = sc.right-INSIDE;        }
  if ( rc->right  <= sc.left   ) { i++; rc->left = sc.left-width+INSIDE;   }
  if ( rc->top    >= sc.bottom ) { i++; rc->top  = sc.bottom-INSIDE;       }
  if ( rc->bottom <= sc.top    ) { i++; rc->top  = sc.top;                 }

  rc->right  = rc->left + width;
  rc->bottom = rc->top  + height;
  return i;
}

/****************************************************************************/
static int MakeName(HWND hWnd,char *s,int maxs,LPCSTR name, char *ext)
/* Jos name != NULL niin s = name, muuten ohjelman nimi -.EXE + ext         */
{
  char *p;
  if ( name ) {
    _fstrncpy(s,name,maxs); s[maxs-1]=0;
    return 0;
  }
  GetModuleFileName(GetWindowInstance(hWnd),s,maxs);
/*  strncpy(s,_argv[0],maxs); s[maxs-1]=0; Ei toimi DLL:ss! */
  p = strstr(s,".exe"); if ( !p ) p = strstr(s,".EXE");
  if ( !p ) return 1;
  strncpy(p,ext,5); p[4]=0;
  return 0;
}

/****************************************************************************/
#define RC wndpl.rcNormalPosition
/****************************************************************************/
/* Makro INI_WNDPL mritt tarvittavat muuttujat ja laittaa niiden        */
/* arvot oikein nykyisen ikkunan paikan mukaan.  Tmn jlkeen joko luetaan */
/* tai kirjoitetaan INI-tiedostoon makrolla INI.  INI pit olla oikein     */
/* mritelty ennen tmn makron kutsua.                                    */
#define INI_WNDPL                                                            \
  int width,height;                                                          \
  char inifilename[80];                                                      \
  WINDOWPLACEMENT wndpl;                                                     \
  MakeName(hWnd,N_S(inifilename),Inifilename,".INI");                        \
                                                                             \
  wndpl.length = sizeof(wndpl); /* Muista!  Muuten ei toimi! */              \
  GetWindowPlacement(hWnd, &wndpl);                                          \
  width  = RC.right-RC.left; height = RC.bottom-RC.top;                      \
                                                                             \
  INI(RC.left,"x");    INI(RC.top,"y");                                      \
  if ( size ) {                                                              \
    INI(width,"leveys"); INI(height,"korkeus");                              \
  }                                                                          \
  INI(wndpl.showCmd,"minmax");                                               \
/****************************************************************************/


/****************************************************************************/
/* Makro INI lukee kokonaisluvun inifilest kohdasta entry.                 */
/* Oletuksena kytetn lukua itsen.                                      */
#define INI(i,entry) i=GetPrivateProfileInt(section,entry,i,inifilename);

/****************************************************************************/
void WINAPI_LIB RestorePos(HWND hWnd,LPCSTR Inifilename,LPCSTR section,
                           tSaveposSizeEnum size)
/* Aliohjelmalla palautetaan ikkuna siihen kokoon, paikkaan ja tilaan, joka
** on talletettu tiedostoon inifilename kohtaan section
** Ongelmia:
**  1. Jos tietoja ei ole tallettu, ei saa muuttaa vastaavaa arvoa!
**     Makro HAE hoitaa tmn
**  2. Jos ikkuna on joutunut nytn ulkopuolella, pitisi se saada nytn
**     sislle.
**  Ratkaisu:  Selvitetn ensin ikkunan nykyhetken tiedot ja sitten
**     kytetn oletusarvoina nykytietoja.  Jos jokin niist puuttuu
**     INI-tiedostosta, j sille nykyinen arvo.
**     Jos ei-lapsi-ikkuna meinaa joutua ruudun ulkopuolelle, pakotetaan se
**     ruudun sislle ainakin hieman.
*/
{
  INI_WNDPL;
  RC.right=RC.left+width; RC.bottom=RC.top+height;

  if ( GetParent(hWnd) == NULL ) ForceInsideScreen(&RC);
//  wndpl.showCmd=SW_HIDE;
  SetWindowPlacement(hWnd, &wndpl);
}
#undef INI

/****************************************************************************/
/* Makro K kirjoittaa annetun kokonaisluvun inifileen kohtaan entry         */
#define INI(i,entry) {char s[20]; wsprintf(s,"%d",i);\
  WritePrivateProfileString(section,entry,s,inifilename);}

/****************************************************************************/
void WINAPI_LIB SavePos(HWND hWnd,LPCSTR Inifilename,LPCSTR section,
                        tSaveposSizeEnum size)
/* Talletetaan ikkunan paikka INI-tiedostoon.
** Onglmia jos kytetn GetWindowRect-kutsua
**   1. Jos ikkuna on minimoitu tai maksimoitu, pit se ensin palauttaa
**      Normaaliin kokoonsa sen paikan mrmikseksi ja sitten palauttaa
**      ikkuna jlleen takaisin.
**   2. Jos ikkuna on lapsi-ikkuna, antaa GetWindowRect sen tiedot
**      nytn koordinaateissa ja MoveWindow taas palautusvaiheessa
**      tarvitsee tiedot Client-koordinaateissa (suhteessa isikkunaan)
**   3. Jos ohjelma on loppumassa (viesti WM_DESTROY), on ikkuna piilotettu.
**      Jos minimoitu ikkuna palautetaan mittaamista varten normaaliin
**      kokoonsa, ja ohjelma on loppumssa, on ikkuna piilotettu.  Se pit
**      siis mys jtt piilotetuksi tmn aliohjelman jlkeen tai siit
**      j nyttn ikkunan haamu ohjelman loppumisen jlkeen.
**   4. Aliohjelmakutsu pit voida tehd mys kesken ohjelman, ilman
**      ett ohjelmasta ollaan postumassa.
** Ratkaisu: Kaikki ongelmat postuvat kun kytetn kutsua
**      GetWindowPlacement
*/
{
  INI_WNDPL;
}
#undef INI